use std::collections::HashMap;

use serde::{Deserialize, Serialize};

use crate::model::geometry::UIntVec2;
use crate::model::icon_descriptor::IconDescriptor;
use crate::model::key_modes;
use crate::model::position::{KeyPosition, KnobPosition};
use crate::model::rgb::RGB8WithOptionalA;

#[derive(Debug, Deserialize)]
pub struct File {
    pub id: Option<String>,
    pub scrolling: Option<ScrollingConfig>,
    pub keys: HashMap<KeyPosition, KeyConfig>,
}

#[derive(Debug)]
pub struct Page {
    pub id: String,
    pub scrolling: Option<ScrollingConfig>,
    pub keys: HashMap<KeyPosition, KeyConfig>,
}

#[derive(Debug, Deserialize)]
pub struct ScrollingConfig {
    pub knob: Option<KnobPosition>,
    pub key: Option<ScrollingKeysConfig>,
    pub delta: u8,
    #[serde(default)]
    pub axis: ScrollingConfigAxis,
}

#[derive(Debug, Deserialize)]
pub struct ScrollingKeysConfig {
    pub previous: UIntVec2,
    pub next: UIntVec2,
}

#[derive(Debug, Default, Eq, PartialEq, Hash, Deserialize)]
#[serde(rename_all = "kebab-case")]
pub enum ScrollingConfigAxis {
    #[default]
    Vertical,
    Horizontal,
}

#[derive(Debug, Default, Clone, PartialEq, Serialize, Deserialize)]
pub struct KeyStyle {
    pub label: Option<String>,
    pub icon: Option<IconDescriptor>,
    pub border: Option<RGB8WithOptionalA>,
}

impl KeyStyle {
    pub fn merge_over(&self, base: &KeyStyle) -> KeyStyle {
        KeyStyle {
            label: self.label.as_ref().or(base.label.as_ref()).cloned(),
            icon: self.icon.as_ref().or(base.icon.as_ref()).cloned(),
            border: self.border.or(base.border),
        }
    }
}

#[derive(Debug, Deserialize)]
pub struct KeyConfig {
    #[serde(default, flatten)]
    pub base_style: KeyStyle,

    #[serde(default)]
    pub mode: KeyModes,
}

#[allow(non_snake_case)]
#[derive(Debug, Default, Deserialize)]
pub struct KeyModes {
    pub vibrate: Option<key_modes::vibrate::Config>,
    pub media__play_pause: Option<key_modes::media::PlayPauseConfig>,
    pub media__previous: Option<key_modes::media::PreviousAndNextConfig>,
    pub media__next: Option<key_modes::media::PreviousAndNextConfig>,
    pub spotify__shuffle: Option<key_modes::spotify::ShuffleConfig>,
    pub spotify__repeat: Option<key_modes::spotify::RepeatConfig>,
    pub home_assistant__switch: Option<key_modes::home_assistant::SwitchConfig>,
    pub home_assistant__button: Option<key_modes::home_assistant::ButtonConfig>,
}

pub type StyleByStateMap<State> = HashMap<State, KeyStyle>;
